"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const inquirer_1 = __importDefault(require("inquirer"));
const fs_1 = __importDefault(require("fs"));
const path_1 = require("path");
const stacks_1 = require("./paths/stacks");
const stacks_2 = require("./questions/stacks");
const file_1 = require("./utils/file");
const rxjs_1 = require("rxjs");
const Injector_1 = __importDefault(require("./utils/Injector"));
const commands_1 = require("./utils/commands");
const messages_1 = require("../utils/messages");
const constants_1 = require("./constants");
const prompts = new rxjs_1.Subject();
let selectedStacks = [];
let selectedFrameworks = [];
const onStacksAnswered = (name, answer) => {
    selectedFrameworks.push(name);
    const extras = stacks_2.frameworkExtrasSelect(name, answer);
    if (extras) {
        prompts.next(extras);
    }
    if (selectedStacks.length === selectedFrameworks.length) {
        prompts.complete();
    }
};
const onEachAnswer = ({ name, answer }) => {
    console.clear();
    switch (name) {
        case "stacks": {
            selectedStacks = answer;
            answer.forEach((stack) => {
                prompts.next(stacks_2.createStackFrameworkSelect(stack));
            });
            break;
        }
        case "backend": {
            onStacksAnswered(name, answer);
            break;
        }
        case "frontend": {
            onStacksAnswered(name, answer);
            break;
        }
        default:
            if (name !== "project-name")
                prompts.complete();
            break;
    }
};
const onError = (err) => console.error(err);
const projectName = {
    name: "project-name",
    type: "input",
    message: "Project name:",
    validate: function (input) {
        try {
            if (/^([A-Za-z\-\_\d])+$/.test(input)) {
                const currentFiles = fs_1.default.readdirSync(process.cwd());
                currentFiles.forEach(f => {
                    if (f === input) {
                        throw new Error("file exists");
                    }
                });
                return true;
            }
            else
                throw new Error("invalid name");
        }
        catch (error) {
            switch (error.message) {
                case "file exists":
                    return messages_1.errorMessage("A directory named " + input + " already exists!");
                case "invalid name":
                    return messages_1.errorMessage("Project name may only include letters, numbers, underscores and hashes.");
                default:
                    return messages_1.errorMessage(error.message);
            }
        }
    },
    default: "chefs-special",
};
const prompt = inquirer_1.default.prompt(prompts);
prompt.ui.process.subscribe(onEachAnswer, onError);
prompt.then((_a) => {
    var { stacks } = _a, answers = __rest(_a, ["stacks"]);
    const projectName = answers["project-name"];
    let stackPaths;
    if (!stacks)
        return;
    stackPaths = stacks_1.createStackPaths(stacks, answers);
    if (!stackPaths)
        return;
    const fullStack = stacks.length === 2;
    fs_1.default.mkdirSync(path_1.join(process.cwd(), projectName));
    stacks.forEach(stack => {
        stackPaths[stack].forEach(({ path, extras, initPath }) => __awaiter(void 0, void 0, void 0, function* () {
            const folderName = stack === "frontend" ? "client" : "server";
            const targetPath = fullStack
                ? path_1.join(process.cwd(), projectName, folderName)
                : path_1.join(process.cwd(), projectName);
            if (path) {
                if (path.includes(constants_1.EMPTY_FOLDER)) {
                    if (fullStack) {
                        fs_1.default.mkdirSync(path_1.join(process.cwd(), projectName, folderName));
                    }
                    return;
                }
                console.log(messages_1.infoMessage(stack + ": Installing core dependencies..."));
                yield file_1.copyContents(path, targetPath);
                commands_1.installPackages(targetPath);
            }
            if (initPath && !process.env.DEV) {
                if (!fs_1.default.existsSync(targetPath)) {
                    fs_1.default.mkdirSync(targetPath);
                }
                console.log(messages_1.infoMessage(stack + ": Installing core dependencies..."));
                const { default: init } = yield Promise.resolve().then(() => __importStar(require(initPath)));
                const initArgs = { folderName: fullStack ? folderName : ".", cwd: targetPath };
                init(initArgs);
            }
            extras.forEach(({ path, initializer, name }) => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    console.log(messages_1.infoMessage(`Installing packages for ${name}`));
                    yield file_1.copyContents(path, targetPath);
                    if (initializer) {
                        yield Injector_1.default.inject({ projectPath: targetPath, referencePath: initializer });
                    }
                    console.log(messages_1.successMessage(`${name} installed`));
                }
                catch (err) {
                    console.log(messages_1.errorMessage(err.message));
                }
            }));
        }));
    });
    if (!(!fullStack &&
        stackPaths["frontend"] &&
        stackPaths["frontend"].filter(({ name }) => name.includes("nextjs") || name.includes("gatsby") || name.includes("create-react-app")).length)) {
        const files = fs_1.default.readdirSync(stacks_1.basePath);
        const includes = [".gitignore", ".prettierrc", "LICENSE"];
        const essentials = files.filter(f => !!includes.filter(_f => _f === f).length);
        essentials.forEach(f => {
            const p = path_1.join(stacks_1.basePath, f);
            const dest = path_1.join(process.cwd(), projectName, f);
            if (!fs_1.default.existsSync(dest))
                fs_1.default.copyFileSync(p, dest);
        });
    }
});
prompts.next(projectName);
prompts.next(stacks_2.selectStacks);
//# sourceMappingURL=index.js.map