"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createStackPaths = exports.stackPath = exports.basePath = void 0;
const path_1 = require("path");
const fs_1 = require("fs");
const constants_1 = require("../constants");
const node_modules_path = "../../../../node_modules";
exports.basePath = path_1.join(__dirname, node_modules_path, "project-chef-templates");
exports.stackPath = (stack) => path_1.join(exports.basePath, stack);
function createStackPaths(stacks, data) {
    const paths = { frontend: [], backend: [] };
    stacks.forEach(stack => {
        const framework = data[stack];
        if (framework === constants_1.EMPTY_FOLDER)
            return;
        const extras = data[stack + "Extras"];
        const templatePath = path_1.join(exports.stackPath(stack), framework, "main");
        const initPath = path_1.join(exports.stackPath(stack), framework, "init");
        const templateExists = fs_1.existsSync(templatePath);
        const initExists = fs_1.existsSync(initPath);
        const extrasPaths = extras
            ? extras.map(e => {
                const base = path_1.join(exports.stackPath(stack), framework, "extras", e);
                const baseDirs = fs_1.readdirSync(base);
                const hasMain = baseDirs.findIndex(x => x === "main") > -1;
                const hasInit = baseDirs.findIndex(x => x === "init") > -1;
                if (!hasMain)
                    throw new Error("main directory for extra package not found!");
                return { name: e, path: path_1.join(base, "main"), initializer: hasInit ? path_1.join(base, "init") : undefined };
            })
            : [];
        paths[stack].push({
            path: templateExists ? templatePath : undefined,
            initPath: initExists ? initPath : undefined,
            name: framework,
            extras: extrasPaths,
        });
    });
    return paths;
}
exports.createStackPaths = createStackPaths;
//# sourceMappingURL=stacks.js.map