"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const path_1 = require("path");
const fs_1 = require("fs");
const misc_1 = require("./misc");
const isDev = !!process.env.DEV;
class Injector {
    constructor({ packages, devPackages, configs, projectPath, referencePath }) {
        this.installAllPackages = () => {
            const commands = [
                this.packages.length ? `yarn --cwd ${this.projectPath} add ${this.packages.join(" ")}` : "",
                this.devPackages.length ? `yarn --cwd ${this.projectPath} add -D ${this.devPackages.join(" ")}` : "",
            ];
            commands.forEach(c => {
                if (c)
                    child_process_1.execSync(c);
            });
        };
        this.handleConfigs = () => __awaiter(this, void 0, void 0, function* () {
            if (!this.configs.length)
                return;
            this.configs.map(({ file, appenders }) => __awaiter(this, void 0, void 0, function* () {
                const targetConfig = yield this.getConfigFromProject(file);
                if (!targetConfig)
                    return;
                const { default: referenceConfig } = yield Promise.resolve().then(() => __importStar(require(path_1.join(this.referencePath, file))));
                if (!appenders.length)
                    return;
                appenders.forEach(field => {
                    let targetField;
                    let referenceField;
                    if (field.split(".").length > 1) {
                        const fields = field.split(".");
                        targetField = misc_1.getNestedFields([...fields], targetConfig);
                        referenceField = misc_1.getNestedFields([...fields], referenceConfig);
                        field = fields[fields.length - 1];
                    }
                    else {
                        targetField = targetConfig;
                        referenceField = referenceConfig;
                    }
                    if (Array.isArray(targetField[field])) {
                        targetField[field] = [...targetField[field], ...referenceConfig[field]];
                    }
                    else {
                        targetField[field] = referenceField[field];
                    }
                });
                fs_1.writeFileSync(path_1.join(this.projectPath, file), JSON.stringify(targetConfig));
            }));
        });
        this.getConfigFromProject = (filename) => __awaiter(this, void 0, void 0, function* () {
            try {
                const configPath = path_1.join(this.projectPath, filename);
                const exists = fs_1.statSync(configPath).isFile();
                if (!exists)
                    return null;
                const { default: config } = yield Promise.resolve().then(() => __importStar(require(configPath)));
                return config;
            }
            catch (err) {
                return null;
            }
        });
        this.packages = packages || [];
        this.devPackages = devPackages || [];
        this.configs = configs || [];
        this.projectPath = projectPath;
        this.referencePath = referencePath;
    }
}
exports.default = Injector;
Injector.loadFromConfig = ({ projectPath, referencePath, }) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = {
            projectPath,
            referencePath,
        };
        const { default: { configs, devPackages, packages }, } = yield Promise.resolve().then(() => __importStar(require(path_1.join(config.referencePath, "injector.config.json"))));
        config.configs = configs || [];
        config.packages = packages || [];
        config.devPackages = devPackages || [];
        const instance = new Injector(config);
        return instance;
    }
    catch (err) {
        console.log(err);
        return undefined;
    }
});
Injector.inject = (config) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const injector = yield Injector.loadFromConfig(config);
        if (!injector) {
            throw new Error("Invalid config for Injector, check your injector.config.json or the object being passed to constructor");
        }
        if (!isDev) {
            injector.installAllPackages();
        }
        injector.handleConfigs();
    }
    catch (error) {
        console.error(error.message);
    }
});
//# sourceMappingURL=Injector.js.map